/*

*/

import renderer.scene.*;
import renderer.framebuffer.*;
import renderer.pipeline.*;

import java.awt.Color;

/**
   Here is a picture of a line segment, from vertex v0
   to vertex v1, that needs to be clipped four times
   before we get the final clipped line segment from
   vertex v4 to vertex v5.
   <pre>{@code
                              v0
                              +
                          |  /
                          | /
                          |/
                          + v2
      x = -1             /|
        |               / |
        |              /  |
     ---+-------------+---+----- y = 1
        |            /v4  |
        |           /     |
        |          /      |
        |         /       |
        |        /        |
        |       /         |
        |      /          |
        |     /           |
        |    /            |
     ---+---+-------------+----- y = -1
        |  /v5            |
        | /               |
        |/                |
        + v3            x = 1
       /|
      / |
     +
    v1

   }</pre>
   <p>
   When the clipping algorithm is finished, it will have
   added four new vertices and four new colors to the vertex
   and color lists of this scene's model. And the single line
   segment in the model will only use two of the six vertices
   and two of the six colors in the model's vertex and color
   lists.
   <p>
   When you run this program you can see the debugging output
   from the clipping algorithm and you can see the new vertices
   and colors being added to the model's vertex and color lists.
*/
public class ExtremeClippingExample_R5
{
   public static void main(String[] args)
   {
      final Scene scene = new Scene();
      final Model model = new Model("extreme clipping example");
      scene.addModel(model);

      model.addVertex(new Vertex( 1.25,  1.5, -1),
                      new Vertex(-1.25, -1.5, -1));

      model.addColor(new Color(255, 0, 0),
                     new Color(0, 0, 255));

      model.addLineSegment(new LineSegment(0, 1, 0, 1));

      Pipeline.debug = true;
      Clip.debug = true;
      System.out.println(model);

      final FrameBuffer fb = new FrameBuffer(100, 100);
      fb.clearFB(Color.darkGray);
      Pipeline.render(scene, fb);
      fb.dumpFB2File("ExtremeClippingExample_R5.ppm");
   }
}
