/*

*/

import renderer.scene.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;
import javax.swing.JFrame;
import java.awt.BorderLayout;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.event.ComponentListener;
import java.awt.event.ComponentEvent;

/**
   This program renders a fan of lines two ways, one way using the
   rasterizer without any antialiasing, and a second way using
   the raterizer that implements a simple antialiasing algorithm.
   Use the 'a' key to switch between the two renderings.

   To see how the antialiasing works, look at the rendered
   images using a pixel magnifying program.
*/
public class AntialiasingDemo_2 implements KeyListener, MouseListener, ComponentListener
{
   private final Scene scene;          // The event handlers need
   private final JFrame jf;            // access to these fields.
   private final FrameBufferPanel fbp;

   private static final int numberOfLines = 30;

   private static Color backgroundColor  = Color.green;
   private static Color foregroundColor1 = Color.red;
   private static Color foregroundColor2 = Color.red; // try making this different

   /**
      This constructor instantiates the Scene object
      and initializes it with appropriate geometry.
      Then this constructor instantiates the GUI.
   */
   public AntialiasingDemo_2()
   {
      // Create the Scene object that we shall render
      scene = new Scene();

      // Create a Model object.
      final Model model = new Model();

      // Add the model to the Scene.
      scene.addModel(model);

      // Create a fan of lines, coming out of (-1, -1, -1), in
      // the square with -1 <= x <= 1, -1 <= y <= 1, and z = -1.
      model.addVertex( new Vertex(-1, -1, -1) );
      model.addColor( foregroundColor1 );
      model.addColor( foregroundColor2 );
      for (int i = 0; i < numberOfLines; i++)
      {
         model.addVertex( new Vertex(1, -1 + 2*(double)i/(double)numberOfLines, -1) );
         model.addLineSegment(new LineSegment(0, 2*i+1, 0, 1));  // slope < 1
         model.addVertex( new Vertex(-1 + 2*(double)i/(double)numberOfLines, 1, -1) );
         model.addLineSegment(new LineSegment(0, 2*i+2, 0, 1));  // slope > 1
      }
      model.addVertex( new Vertex(1, 1, -1) );
      model.addLineSegment(new LineSegment(0, 2*numberOfLines+1, 0, 1)); // 45 degree line


      // Define initial dimensions for a FrameBuffer.
      final int width  = 900;
      final int height = 900;

      // Create a FrameBufferPanel that will hold a FrameBuffer.
      fbp = new FrameBufferPanel(width, height);

      // Create a JFrame that will hold the FrameBufferPanel.
      jf = new JFrame("Renderer 2");
      jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      jf.setLocationByPlatform(true);
      // Place the FrameBufferPanel in the JFrame.
      jf.getContentPane().add(fbp, BorderLayout.CENTER);
      jf.pack();
      jf.setVisible(true);

      // Register this object as the event listener for JFrame events.
      jf.addKeyListener(this);
      jf.addMouseListener(this);
      jf.addComponentListener(this);
   }


   // Implement the KeyListener interface.
   @Override public void keyPressed(KeyEvent e){}
   @Override public void keyReleased(KeyEvent e){}
   @Override public void keyTyped(KeyEvent e)
   {
      //System.out.println( e );

      char c = e.getKeyChar();
      if ('h' == c)
      {
         print_help_message();
         return;
      }
      else if ('d' == c)
      {
         Pipeline.debug = ! Pipeline.debug;
         Clip.debug = ! Clip.debug;
         RasterizeAntialias.debug = ! RasterizeAntialias.debug;
      }
      else if ('a' == c)
      {
         RasterizeAntialias.doAntialiasing = ! RasterizeAntialias.doAntialiasing;
         System.out.print("Anti-aliasing is turned ");
         System.out.println(RasterizeAntialias.doAntialiasing ? "On" : "Off");
      }
      else if ('g' == c)
      {
         RasterizeAntialias.doGamma = ! RasterizeAntialias.doGamma;
         System.out.print("Gamma correction is turned ");
         System.out.println(RasterizeAntialias.doGamma ? "On" : "Off");
      }
      else if ('c' == c)
      {
         // Change the solid random color of the cube.
         ModelShading.setRandomColor(scene.getModel(0));
      }
      else if ('C' == c)
      {
         // Change each color in the cube to a random color.
         ModelShading.setRandomColors(scene.getModel(0));
      }
      else if ('e' == c && e.isAltDown())
      {
         // Change the random color of each vertex of the model.
         ModelShading.setRandomVertexColors(scene.getModel(0));
      }
      else if ('e' == c)
      {
         // Change the solid random color of each edge of the model.
         ModelShading.setRandomLineSegmentColors(scene.getModel(0));
      }
      else if ('E' == c)
      {
         // Change the random color of each end of each edge of the model.
         ModelShading.setRainbowLineSegmentColors(scene.getModel(0));
      }
      else if ('b' == c)
      {
         // Change the background color.
         backgroundColor = ModelShading.randomColor();
      }

      // Render again.
      FrameBuffer fb = fbp.getFrameBuffer();
      fb.vp.setBackgroundColorVP(backgroundColor);
      fb.vp.clearVP();
      Pipeline.render(scene, fb.vp);
      fbp.update();
      jf.repaint();
   }


   // Implement the MouseListener interface.
   @Override public void mousePressed(MouseEvent e){}
   @Override public void mouseReleased(MouseEvent e){}
   @Override public void mouseEntered(MouseEvent e){}
   @Override public void mouseExited(MouseEvent e){}
   @Override public void mouseClicked(MouseEvent e)
   {
      //System.out.println( e );

      if (! e.isControlDown())
      {
         RasterizeAntialias.doAntialiasing = ! RasterizeAntialias.doAntialiasing;
         System.out.print("Anti-aliasing is turned ");
         System.out.println(RasterizeAntialias.doAntialiasing ? "On" : "Off");
      }
      else if (e.isControlDown())
      {
         RasterizeAntialias.doGamma = ! RasterizeAntialias.doGamma;
         System.out.print("Gamma correction is turned ");
         System.out.println(RasterizeAntialias.doGamma ? "On" : "Off");
      }

      // Render again.
      FrameBuffer fb = fbp.getFrameBuffer();
      fb.vp.setBackgroundColorVP(backgroundColor);
      fb.vp.clearVP();
      Pipeline.render(scene, fb.vp);
      fbp.update();
      jf.repaint();
   }


   // Implement the ComponentListener interface.
   @Override public void componentMoved(ComponentEvent e){}
   @Override public void componentHidden(ComponentEvent e){}
   @Override public void componentShown(ComponentEvent e){}
   @Override public void componentResized(ComponentEvent e)
   {
      //System.out.println( e );

      // Get the new size of the FrameBufferPanel.
      int w = fbp.getWidth();
      int h = fbp.getHeight();

      // Create a new FrameBuffer that fits the FrameBufferPanel.
      FrameBuffer fb = new FrameBuffer(w, h);
      fbp.setFrameBuffer(fb);
      fb.vp.setBackgroundColorVP(backgroundColor);
      fb.vp.clearVP();
      Pipeline.render(scene, fb.vp);
      fbp.update();
      jf.repaint();
   }


   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      print_help_message();

      // We need to call the program's constructor in the
      // Java GUI Event Dispatch Thread, otherwise we get a
      // race condition between the constructor (running in
      // the main() thread) and the very first ComponentEvent
      // (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         () -> {new AntialiasingDemo_2();}
      );
   }//main()


   private static void print_help_message()
   {
      System.out.println("Use the 'd' key to toggle debugging information on and off.");
      System.out.println("Use the 'a' key to toggle antialiasing on and off.");
      System.out.println("Use the 'g' key to toggle gamma correction on and off.");
      System.out.println("Use mouse click to toggle antialiasing on and off.");
      System.out.println("Use mouse control-click to toggle gamma correction on and off.");
      System.out.println("Use the 'c' key to change the random line color.");
      System.out.println("Use the 'C' key to randomly change model's colors.");
    //System.out.println("Use the 'e' key to change the random vertex colors.");
      System.out.println("Use the 'e' key to change the random solid edge colors.");
      System.out.println("Use the 'E' key to change the random edge colors.");
      System.out.println("Use the 'b' key to change the random background color.");
      System.out.println("Use the 'h' key to redisplay this help message.");
   }
}
